/**
 * This file holds the credit card plugin functionalities. 
 */

package mf.plugins.MfCreditCard;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JOptionPane;

import org.mozilla.javascript.annotations.JSGetter;
import org.mozilla.javascript.annotations.JSSetter;

import com.servoy.j2db.documentation.ServoyDocumented;
import com.servoy.j2db.scripting.IReturnedTypesProvider;
import com.servoy.j2db.scripting.IScriptable;
import com.servoy.j2db.scripting.annotations.JSReadonlyProperty;
import com.servoy.j2db.util.Debug;

/**
 * @author prasady
 * 
 * @since 08/20/2013
 *
 */
@ServoyDocumented(publicName="MFCreditCardPlugin", scriptingName="plugins.MFCreditCardPlugin")
public class MFCreditCardPluginProvider implements IScriptable, 
		IReturnedTypesProvider {
	
	//Initializing variables
	public String firstName = "";
	public String lastName = "";
	public String address = "";
	public String city = "";
	public String state = "";
	public String country = "";
	public String zip = "";
	
	@JSGetter
	public String js_getFirstName() {
		
		return firstName;							//Gets the first name of the payee
	}
	
	@JSSetter
	public void js_setFirstName(String firstName) {
		
		this.firstName = firstName;					//Sets the first name of the payee
	}
	
	@JSGetter
	public String js_getLastName() {
		
		return lastName;							//Gets the last name of the payee
	}
	
	@JSSetter
	public void js_setLastName(String lastName) {
		
		this.lastName = lastName;					//Sets the last name of the payee
	}
	
	@JSGetter
	public String js_getAddress() {
		
		return address;								//Gets the address of the payee
	}
	
	@JSSetter
	public void js_setAddress(String address) {
		
		this.address = address;						//Sets the address of the payee
	}
	
	@JSGetter
	public String js_getCity() {
		
		return city;								//Gets the city name of the payee
	}
	
	@JSSetter
	public void js_setCity(String city) {
		
		this.city = city;							//Sets the city name of the payee
	}

	@JSGetter
	public String js_getState() {
		
		return state;								//Gets the state name of the payee
	}
	
	@JSSetter
	public void js_setState(String state) {
		
		this.state = state;							//Sets the state name of the payee
	}
	
	@JSGetter
	public String js_getCountry() {
		
		return country;								//Gets the country name of the payee
	}
	
	@JSSetter
	public void js_setCountry(String country) {
		
		this.country = country;						//Sets the country name of the payee
	}
	
	@JSGetter
	public String js_getZip() {
		
		return zip;									//Gets the zip code of the payee
	}
	
	@JSReadonlyProperty
	public String js_getPluginVersion() {
		
		return "Version 1.0";						//Gets the version number of the plugin
	}
	
	@JSSetter
	public void js_setZip(String zip) {
		
		this.zip = zip;
	}

	/**
	 * This function connects to 'Authorized.net' payment gateway for credit card payment and returns the response of respective transaction.
	 *
	 * @sample
	 * //This function connects to 'Authorized.net' payment gateway for credit card payment
	 * //and returns the response of respective transaction.
	 * //Parameters for payment function are Merchant login id, transaction id, version number,credit card number,
	 * //expiry date of the card, CCV/CVV number(The three- or four-digit number on the back of a credit card (on the front for American Express)),
	 * //amount and transaction name.
	 * //This function returns a response object
	 * //which holds response code, response description, transaction name, amount values
	 * //(can view in solution explorer when you select Response( Under MFCreditCardPlugin))
	 * //response code.. 1 .. gives you a successful transaction
	 * //For more information on response codes, please refer
	 * //http://www.authorize.net/support/merchant/Transaction_Response/Response_Reason_Codes_and_Response_Reason_Text.htm
	 * /** @type {Object} *&#47;
	 * var resp = plugins.MFCreditCardPlugin.payment('35yJ2Qc2','3w6vqjCz29t4P4C9','3.1','6299991111111111', '0115', '1', '123', 'Sample Transaction');
	 * var respCode = resp.getResponseCode();
	 * var respDesc = resp.getResponseDesc();
	 * var tranName = resp.getTransName();
	 * var amount = resp.getAmount();
	 * 
	 * @param merLgnKey Merchant Login Key provided by Authorize.net
	 * @param trancKey  Merchant transaction Key provided by Authorize.net
	 * @param ver 		Version number provided by Authorize.net
	 * @param cardNo	Credit Card number
	 * @param expDate	Expiry date of the card
	 * @param cvvNo		CVV/CCV number
	 * @param amount	Amount for transaction
	 * @param tranDesc  Transaction name
	 * 
	 * @return Response Object the Response object
	 * 
	 */
	@SuppressWarnings("unchecked")
	public Response js_processPayment(String merLgnKey, String trancKey, String ver, String cardNo, 
								String expDate, String cvvNo, String amount, String tranDesc)
	{
		
		// By default, this sample code is designed to post to our test server for
		// developer accounts: https://test.authorize.net/gateway/transact.dll
		// for real accounts (even in test mode), please make sure that you are
		// posting to: https://secure.authorize.net/gateway/transact.dll
		URL post_url;
		String responseData = "";
		String [] responses = {};
		
		try {
			
			//Connects to Authorize.net secure payment gateway
			post_url = new URL("https://secure.authorize.net/gateway/transact.dll");
			@SuppressWarnings("rawtypes")
			Hashtable post_values = new Hashtable();
			  
			// the API Login ID and Transaction Key must be replaced with valid values
			if( merLgnKey == null || merLgnKey == "")
				JOptionPane.showMessageDialog(null, "Merchant Login Key cannot be null", "Ok", 0);
			else
				post_values.put ("x_login", merLgnKey);
			
			if(trancKey == null || trancKey == "")
				JOptionPane.showMessageDialog(null, "Transaction Key cannot be null", "Ok", 0);
			else
				post_values.put ("x_tran_key", trancKey);
			  
			if(ver == null || ver == "")
				JOptionPane.showMessageDialog(null, "Version Key cannot be null", "Ok", 0);
			else
				post_values.put ("x_version", ver);
				
			post_values.put ("x_delim_data", "TRUE");
			post_values.put ("x_delim_char", ",");
			post_values.put ("x_relay_response", "FALSE");
			post_values.put ("x_type", "AUTH_CAPTURE");
			post_values.put ("x_method", "CC");
			post_values.put ("x_card_num", cardNo);
			post_values.put ("x_exp_date", expDate);
			post_values.put ("x_card_code", cvvNo);
			post_values.put ("x_amount", amount);
			post_values.put ("x_description", tranDesc);
			post_values.put ("x_duplicate_window", "60");
			
			if(firstName != null)
				post_values.put ("x_first_name", firstName);
			if(lastName != null)
				post_values.put ("x_last_name", lastName);
			if(address != null)
				post_values.put ("x_address", address);
			if(city != null)
				post_values.put ("x_city", city);
			if(state != null)
				post_values.put ("x_state", state);
			if(country != null)
				post_values.put ("x_country", country);
			if(zip != null)
				post_values.put ("x_zip", zip);
			
			// Additional fields can be added here as outlined in the AIM integration
			// guide at: http://developer.authorize.net
	
			// This section takes the input fields and converts them to the proper format
			// for an http post.  For example: "x_login=username&x_tran_key=a1B2c3D4"
			
			StringBuffer post_string = new StringBuffer();
			@SuppressWarnings("rawtypes")
			Enumeration keys = post_values.keys();
			while( keys.hasMoreElements() ) {
				
			  String key = URLEncoder.encode(keys.nextElement().toString(),"UTF-8");
			  String value = URLEncoder.encode(post_values.get(key).toString(),"UTF-8");
			  post_string.append(key + "=" + value + "&");
			}
	
			// Open a URLConnection to the specified post url
			URLConnection connection = post_url.openConnection();
			connection.setDoOutput(true);
			connection.setUseCaches(false);
	
			// this line is not necessarily required but fixes a bug with some servers
			connection.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
	
			// submit the post_string and close the connection
			DataOutputStream requestObject = new DataOutputStream( connection.getOutputStream() );
			requestObject.write(post_string.toString().getBytes());
			requestObject.flush();
			requestObject.close();
			// process and read the gateway response
			BufferedReader rawResponse = new BufferedReader(new InputStreamReader(connection.getInputStream()));
			
			@SuppressWarnings("unused")
			String line;
			responseData = rawResponse.readLine();
			responses = responseData.split(",");
			System.out.println(responseData);
			rawResponse.close();
			
			Response resp = new Response();
			
			String respCode = responses[0];
			String respDesc = responses[3];
			String tranName = responses[8];
			String amt = responses[9];
			
			resp.setResponseCode(respCode);
			resp.setResponseDesc(respDesc);
			resp.setTransName(tranName);
			resp.setAmount(amt);
			
			return resp;
		} 
		catch (Exception e) {
			
			e.printStackTrace();
			Debug.error(e);
			
			return null;
		}
			
	}
	
	/**
	 * This function connects to 'Authorized.net' payment gateway for credit card payment and returns the response of respective transaction.
	 *
	 * @sample
	 * //This function connects to 'Authorized.net' payment gateway for credit card payment
	 * //and returns the response of respective transaction.
	 * //Parameters for payment function are Merchant login id, transaction id, version number,credit card number,
	 * //expiry date of the card, CCV/CVV number(The three- or four-digit number on the back of a credit card (on the front for American Express)),
	 * //amount and transaction name.
	 * //This function returns a response object
	 * //which holds response code, response description, transaction name, amount values
	 * //(can view in solution explorer when you select Response( Under MFCreditCardPlugin))
	 * //response code.. 1 .. gives you a successful transaction
	 * //For more information on response codes, please refer
	 * //http://www.authorize.net/support/merchant/Transaction_Response/Response_Reason_Codes_and_Response_Reason_Text.htm
	 * /** @type {Object} *&#47;
	 * var resp = plugins.MFCreditCardPlugin.payment('35yJ2Qc2','3w6vqjCz29t4P4C9','3.1','6299991111111111', '0115', '1', '123', 'Sample Transaction');
	 * var respCode = resp.getResponseCode();
	 * var respDesc = resp.getResponseDesc();
	 * var tranName = resp.getTransName();
	 * var amount = resp.getAmount();
	 * 
	 * @param merLgnKey Merchant Login Key provided by Authorize.net
	 * @param trancKey  Merchant transaction Key provided by Authorize.net
	 * @param ver 		Version number provided by Authorize.net
	 * @param cardNo	Credit Card number
	 * @param expDate	Expiry date of the card
	 * @param cvvNo		CVV/CCV number
	 * @param amount	Amount for transaction
	 * 
	 * @return Response Object the Response object
	 * 
	 */
	public Response js_processPayment(String merLgnKey, String trancKey, String ver, String cardNo, 
			String expDate, String cvvNo, String amount) {
		
		try {
			
			return js_processPayment(merLgnKey, trancKey, ver, cardNo, expDate, cvvNo, amount, null);
		}catch(Exception e) {
			
			e.printStackTrace();
			Debug.error(e);
			
			return null;
		}
	}

	@Override
	public Class<?>[] getAllReturnedTypes() {
		
		return new Class[] { Response.class };
	  
	}

}
