/**
 * This file holds the response functionalities of the current transaction. 
 */

package mf.plugins.MfCreditCard;


import com.servoy.j2db.documentation.ServoyDocumented;
import com.servoy.j2db.scripting.IJavaScriptType;
import com.servoy.j2db.scripting.IScriptable;

/**
 * @author prasady
 * 
 * @since 08/20/2013
 *
 */

@ServoyDocumented
public class Response implements IScriptable,IJavaScriptType {
	
	private String responseCode;
	private String responseDesc;
	private String transactionName;
	private String amount;
	
	/**
	 * Gets the response code of current transaction
	 *
	 * @sample
	 * var resp = plugins.MFCreditCardPlugin.payment('35yJ2Qc2','3w6vqjCz29t4P4C9','3.1','6299991111111111', '0115', '1', '123', 'Sample Transaction');
	 * var respCode = resp.getResponseCode();
	 *
	 * @return Response Code of current transcation
	 */
	public String js_getResponseCode() {
		
		return this.responseCode;
	}
	
	public void setResponseCode(String respCode) {
		
		this.responseCode = respCode;				//Sets the response code
	}
	
	/**
	 * Gets the response description of current transaction
	 *
	 * @sample
	 * var resp = plugins.MFCreditCardPlugin.payment('35yJ2Qc2','3w6vqjCz29t4P4C9','3.1','6299991111111111', '0115', '1', '123', 'Sample Transaction');
	 * var respCode = resp.getResponseDesc();
	 *
	 * @return Response Description of current transaction
	 */
	public String js_getResponseDesc() {
		
		return this.responseDesc;
	}
	
	public void setResponseDesc(String respDesc) {
		
		this.responseDesc = respDesc;				//Sets the response description
	}
	
	/**
	 * Gets the name of current transaction
	 *
	 * @sample
	 * var resp = plugins.MFCreditCardPlugin.payment('35yJ2Qc2','3w6vqjCz29t4P4C9','3.1','6299991111111111', '0115', '1', '123', 'Sample Transaction');
	 * var respCode = resp.getTransName();
	 *
	 * @return Current transaction name if provided
	 */
	public String js_getTransName() {
		
		return this.transactionName;
	}
	
	public void setTransName(String tranName) {
		
		this.transactionName = tranName;			//Sets the transaction name
	}
	
	/**
	 * Gets the transaction amount
	 *
	 * @sample
	 * var resp = plugins.MFCreditCardPlugin.payment('35yJ2Qc2','3w6vqjCz29t4P4C9','3.1','6299991111111111', '0115', '1', '123', 'Sample Transaction');
	 * var respCode = resp.getAmount();
	 *
	 * @return Transaction amount
	 */
	public String js_getAmount() {
		
		return this.amount;
	}
	
	public void setAmount(String amt) {
		
		this.amount = amt;							//Sets the transaction amount
		
	}
}
